#ifndef LIBGSLOOP_H
#define LIBGSLOOP_H

#include <inttypes.h>
#include <sys/poll.h>

struct gsloop_evnet;
struct gsloop_attr;


typedef struct {
	char		_data[168];
} gsloop_t;

typedef struct {
	char		_data[96];
} gsloop_server_t;

typedef struct {
	char		_data[56];
} gsloop_client_t;

typedef struct {
	char		_data[32];
} gsloop_common_t;


typedef struct	gsloop_attr			gsloop_attr_t;
typedef struct	gsloop_server_attr		gsloop_server_attr_t;
typedef struct	gsloop_event			gsloop_event_t;

typedef int	(*gsloop_server_close_fn_t)	(gsloop_server_t	*gsloop_server);
typedef int	(*gsloop_client_event_fn_t)	(gsloop_client_t	*gsloop_client,
						 gsloop_event_t		*gsloop_event,
						 void			*data);
typedef int	(*gsloop_client_attach_fn_t)	(gsloop_client_t	*client,
						 void			*data);


struct gsloop_event {
	int				fd;
	uint32_t			revents;
};

struct gsloop_attr {
	int				max_fd;
};

struct gsloop_server_attr {
	unsigned int			max_clients;

	gsloop_client_event_fn_t	client_event;		/* client's function, called when client is ready */
	void				*client_event_data;	/* arg passed to the client's event fn */

	gsloop_client_attach_fn_t	client_attach;		/* called when client is attached */
	void				*client_attach_data;	/* arg passed to the server's client attach fn */
};


extern int gsloop_init			(gsloop_t			*gsloop,
					 gsloop_attr_t			*attr);

extern int gsloop_destroy		(gsloop_t			*gsloop);

extern int gsloop_dispatch		(gsloop_t			*gsloop,
					 unsigned int			timeout);


extern int gsloop_tcp_server_create	(gsloop_t			*gsloop,
					 uint16_t			port,
					 gsloop_server_attr_t		*attr);


extern int gsloop_client_close		(gsloop_client_t		*gsloop_client);

extern int gsloop_client_set_event_data	(gsloop_client_t		*client_anon,
					 void				*event_data);

extern int gsloop_client_get_fd		(gsloop_client_t		*client_anon);

#endif	/* !LIBGSLOOP_H */
